@@ -22,7 +22,7 @@
 #'   or \code{"jax"} for 'JAX' arrays. Default is \code{"R"}.
 #' @param verbose A logical value indicating whether to print progress information. Default is \code{TRUE}. 
 #' @param conda_env A character string specifying the name of the conda environment to use 
-#'   via \code{reticulate}. Default is \code{"fastrerandomize"}.
+#'   via \code{reticulate}. Default is \code{"fastrerandomize_env"}.
 #' @param conda_env_required A logical indicating whether the specified conda environment 
 #'   must be strictly used. If \code{TRUE}, an error is thrown if the environment is not found. 
 #'   Default is \code{TRUE}.
@@ -84,7 +84,7 @@ generate_randomizations <- function(n_units,
                                    file = NULL, 
                                    return_type = "R", 
                                    verbose = TRUE,
-                                   conda_env = "fastrerandomize", 
+                                   conda_env = "fastrerandomize_env", 
                                    conda_env_required = TRUE
                                    ){
   if (is.null(check_jax_availability(conda_env=conda_env))) { return() }
@@ -99,25 +99,27 @@ generate_randomizations <- function(n_units,
         candidate_randomizations <- fastrerandomize::generate_randomizations_exact(
                                                 n_units = n_units,
                                                 n_treated = n_treated,
-                                                X = X, 
-                                                randomization_accept_prob = randomization_accept_prob, 
-                                                threshold_func = threshold_func, 
-                                                verbose = verbose, 
-                                                conda_env = conda_env, 
+                                                X = X,
+                                                randomization_accept_prob = randomization_accept_prob,
+                                                approximate_inv = approximate_inv,
+                                                threshold_func = threshold_func,
+                                                verbose = verbose,
+                                                conda_env = conda_env,
                                                 conda_env_required = conda_env_required)
     } else if (randomization_type == "monte_carlo"){
         if (verbose){
             message("Using monte carlo randomization")
         }
         candidate_randomizations <- fastrerandomize::generate_randomizations_mc(
-                                                                    n_units = n_units, 
-                                                                    n_treated = n_treated, 
-                                                                    X = X, 
-                                                                    randomization_accept_prob = randomization_accept_prob, 
-                                                                    threshold_func = threshold_func, 
-                                                                    max_draws = max_draws, 
-                                                                    batch_size = batch_size, 
-                                                                    conda_env = conda_env, 
+                                                                    n_units = n_units,
+                                                                    n_treated = n_treated,
+                                                                    X = X,
+                                                                    randomization_accept_prob = randomization_accept_prob,
+                                                                    threshold_func = threshold_func,
+                                                                    max_draws = max_draws,
+                                                                    batch_size = batch_size,
+                                                                    approximate_inv = approximate_inv,
+                                                                    conda_env = conda_env,
                                                                     conda_env_required = conda_env_required,
                                                                     verbose = verbose)
     } else {
