@@ -23,7 +23,7 @@
 #'   statistic.
 #' @param verbose A logical value indicating whether to print progress information. Default is \code{TRUE}.
 #' @param conda_env A character string specifying the name of the conda environment to use 
-#'   via \code{reticulate}. Default is "fastrerandomize".
+#'   via \code{reticulate}. Default is "fastrerandomize_env".
 #' @param conda_env_required A logical indicating whether the specified conda environment 
 #'   must be strictly used. If \code{TRUE}, an error is thrown if the environment is not found. 
 #'   Default is TRUE.
@@ -82,7 +82,7 @@ generate_randomizations_exact <- function(
                                    approximate_inv = TRUE, 
                                    threshold_func = NULL,
                                    verbose = TRUE,
-                                   conda_env = "fastrerandomize", 
+                                   conda_env = "fastrerandomize_env", 
                                    conda_env_required = TRUE){
   if(is.null(check_jax_availability(conda_env=conda_env))) { return(NULL) }
   
@@ -92,8 +92,8 @@ generate_randomizations_exact <- function(
   if(is.null(threshold_func)){ threshold_func <- fastrr_env$VectorizedFastHotel2T2 }
   
   max_rand_num <- choose(n_units, n_treated)
-  assertthat::assert_that( max_rand_num*min(randomization_accept_prob) > 10, 
-              msg = "Value of min(randomization_accept_prob) indices less than 10 accepted randomizations. Increase min(randomization_accept_prob)!")
+  assertthat::assert_that( max_rand_num * randomization_accept_prob > 10,
+              msg = "Value of randomization_accept_prob results in fewer than 10 accepted randomizations. Increase randomization_accept_prob!")
   
   # Get all combinations of positions to set to 1
   combinations <- fastrr_env$jnp$array(  utils::combn(n_units, n_treated) - 1L )
@@ -104,18 +104,22 @@ generate_randomizations_exact <- function(
     # Set up sample sizes for treatment/control
     n0_array <- fastrr_env$jnp$array(  (n_units - n_treated) )
     n1_array <- fastrr_env$jnp$array(  n_treated )
-    
-    # pre-compute matrix inverse 
+
+    # pre-compute matrix inverse
     {
       SAMP_COV_INV_APPROX <- fastrr_env$jnp$reciprocal( fastrr_env$jnp$var( fastrr_env$jnp$array(as.matrix(X)), axis = 0L) )
       {
-        SAMP_COV_INV <-  fastrr_env$jnp$cov( fastrr_env$jnp$array(as.matrix(X)), rowvar = FALSE) 
+        SAMP_COV_INV <-  fastrr_env$jnp$cov( fastrr_env$jnp$array(as.matrix(X)), rowvar = FALSE)
         IS_METAL_BACKEND <- grepl(reticulate::py_str( fastrr_env$jax$devices()[[1]] ), pattern = "METAL")
-        if(IS_METAL_BACKEND){ 
+        if(IS_METAL_BACKEND){
           SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("cpu")[[1]])
         }
+        # Handle single covariate case: cov returns scalar, inv needs 2D
+        if(ncol(X) == 1L) {
+          SAMP_COV_INV <- fastrr_env$jnp$reshape(SAMP_COV_INV, c(1L, 1L))
+        }
         SAMP_COV_INV <- fastrr_env$jnp$linalg$inv(SAMP_COV_INV)
-        if(IS_METAL_BACKEND){ 
+        if(IS_METAL_BACKEND){
           SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("METAL")[[1]])
         }
       }
@@ -138,10 +142,10 @@ generate_randomizations_exact <- function(
       fastrr_env$jnp$array(randomization_accept_prob)
     )
 
-    # Keep only randomizations with balance measure below threshold
+    # Keep only randomizations with balance measure at or below threshold (use less_equal to include ties)
     candidate_randomizations <- fastrr_env$jnp$take(
       candidate_randomizations,
-      indices = (takeM_ <- fastrr_env$jnp$where(fastrr_env$jnp$less(M_results,a_threshold))[[1]] ),
+      indices = (takeM_ <- fastrr_env$jnp$where(fastrr_env$jnp$less_equal(M_results, a_threshold))[[1]] ),
       axis = 0L
     )
     M_results <- fastrr_env$jnp$take( M_results, indices = takeM_, axis = 0L )
