@@ -17,7 +17,7 @@
 #'   Default is \code{TRUE}.
 #' @param verbose A logical value indicating whether to print detailed information about batch processing progress, and GPU memory usage. Default is \code{FALSE}. 
 #' @param conda_env A character string specifying the name of the conda environment to use 
-#'   via \code{reticulate}. Default is \code{"fastrerandomize"}.
+#'   via \code{reticulate}. Default is \code{"fastrerandomize_env"}.
 #' @param conda_env_required A logical indicating whether the specified conda environment 
 #'   must be strictly used. If \code{TRUE}, an error is thrown if the environment is not found. 
 #'   Default is \code{TRUE}.
@@ -76,7 +76,7 @@ generate_randomizations_mc <- function(n_units,
                                        batch_size = 1000, 
                                        approximate_inv = TRUE,
                                        verbose = TRUE,
-                                       conda_env = "fastrerandomize", 
+                                       conda_env = "fastrerandomize_env", 
                                        conda_env_required = TRUE
                                       ){
   if (is.null(check_jax_availability(conda_env=conda_env))) { return(NULL) }
@@ -106,20 +106,24 @@ generate_randomizations_mc <- function(n_units,
   # Convert X to JAX array (float16 can cause issues with matrix inverse)
   X_jax <- fastrr_env$jnp$array(as.matrix(X), dtype = fastrr_env$jnp$float32)
   
-  # pre-compute matrix inverse 
+  # pre-compute matrix inverse
   {
     SAMP_COV_INV_APPROX <- fastrr_env$jnp$reciprocal( fastrr_env$jnp$var( fastrr_env$jnp$array(as.matrix(X)), axis = 0L) )
-  {
-    SAMP_COV_INV <-  fastrr_env$jnp$cov( fastrr_env$jnp$array(as.matrix(X)), rowvar = FALSE) 
-    IS_METAL_BACKEND <- grepl(reticulate::py_str( fastrr_env$jax$devices()[[1]] ), pattern = "METAL")
-    if(IS_METAL_BACKEND){ 
-      SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("cpu")[[1]])
+    {
+      SAMP_COV_INV <-  fastrr_env$jnp$cov( fastrr_env$jnp$array(as.matrix(X)), rowvar = FALSE)
+      IS_METAL_BACKEND <- grepl(reticulate::py_str( fastrr_env$jax$devices()[[1]] ), pattern = "METAL")
+      if(IS_METAL_BACKEND){
+        SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("cpu")[[1]])
+      }
+      # Handle single covariate case: cov returns scalar, inv needs 2D
+      if(ncol(X) == 1L) {
+        SAMP_COV_INV <- fastrr_env$jnp$reshape(SAMP_COV_INV, c(1L, 1L))
+      }
+      SAMP_COV_INV <- fastrr_env$jnp$linalg$inv(SAMP_COV_INV)
+      if(IS_METAL_BACKEND){
+        SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("METAL")[[1]])
+      }
     }
-    SAMP_COV_INV <- fastrr_env$jnp$linalg$inv(SAMP_COV_INV)
-    if(IS_METAL_BACKEND){ 
-      SAMP_COV_INV <- SAMP_COV_INV$to_device(fastrr_env$jax$devices("METAL")[[1]])
-    }
-  }
   }
 
   # Set up sample sizes for treatment/control
@@ -173,8 +177,8 @@ generate_randomizations_mc <- function(n_units,
   })
   
   AllKeys <- fastrr_env$jax$random$split(key, as.integer(max_draws))
-  AllKeySelectionIndices <- split( as.integer( (1L:max_draws) - 1L), 
-                                   sapply(1:num_batches, function(x){rep(x,times = batch_size)}))
+  batch_ids <- ceiling(seq_len(max_draws) / batch_size)
+  AllKeySelectionIndices <- split(as.integer((1L:max_draws) - 1L), batch_ids)
   top_M_results <- sapply(1L:num_batches, function(b_){ # note: vmapping this causes unacceptable memory overhead 
     if(verbose){ message(paste0("On batch ", b_, " of ", num_batches)) }
     top_M_results_ <- top_M_fxn( fastrr_env$jnp$take(AllKeys,
